AndBuild: [&&,&,and]
OrBuild: [||,|, or]
XorBuild: [^, xor]
EqualBuild: ['=','==',equal, eq]
BiggerBuild: ['>', bigger,bg]
BiggerEqualBuild: ['>=', be]
LittlerBuild: ['<', littler,lt]
LittlerEqualBuild: ['<=',le]
PatternBuild:[pt]
InBuild: [in]
NotBuild: [!, not]
ValBuild: [c, val, const]