#coding=utf-8
#author:
from .dirz import dirs, FileDeal
from .lsf import lists, search, searchd,searchs,_search
from .fio import fread, freads, fwrite, fwrites, read,reads,write,writes, makefdir, dirpath,makedir,removes, cover, fcover, sread, swrite, is_abs, xors, fxors, coverx, covers, fcovers
from .fhs import fhash, fhashs
from .csv import to_csv, from_csv, merge_in

__author__ = "Zzz, emails: 1174534295@qq.com, 1309458652@qq.com"
